@@ -233,7 +233,7 @@ print_item* parse_percent_item(const char** str)
 	static const char *short_hash = "CMHTGWRAE";
 	static const char *short_other = "Llpfus";
 	static const unsigned hash_ids[] = {
-		RHASH_CRC32, RHASH_MD5, RHASH_SHA1, RHASH_TTH, RHASH_GOST,
+		RHASH_CRC32, RHASH_MD5, RHASH_SHA1, RHASH_TTH, RHASH_GOST12_256,
 		RHASH_WHIRLPOOL, RHASH_RIPEMD160, RHASH_AICH, RHASH_ED2K
 	};
 	static const unsigned other_flags[] = {
@@ -431,7 +431,7 @@ void print_line(FILE* out, print_item* list, struct file_info *info)
 				| (list->flags & PRINT_FLAG_BASE32 ? RHPR_BASE32 : 0)
 				| (list->flags & PRINT_FLAG_BASE64 ? RHPR_BASE64 : 0)
 				| (list->flags & PRINT_FLAG_HEX ? RHPR_HEX : 0);
-			if ((hash_id == RHASH_GOST || hash_id == RHASH_GOST_CRYPTOPRO) && (opt.flags & OPT_GOST_REVERSE))
+			if ((hash_id == RHASH_GOST94 || hash_id == RHASH_GOST94_CRYPTOPRO) && (opt.flags & OPT_GOST_REVERSE))
 				print_flags |= RHPR_REVERSE;
 
 			len = rhash_print(buffer, info->rctx, hash_id, print_flags);
@@ -514,29 +514,32 @@ void free_print_list(print_item* list)
  */
 void init_hash_info_table(void)
 {
-	unsigned index, bit;
+	unsigned bit;
 	unsigned short_opt_mask = RHASH_CRC32 | RHASH_MD5 | RHASH_SHA1 | RHASH_TTH | RHASH_ED2K |
-		RHASH_AICH | RHASH_WHIRLPOOL | RHASH_RIPEMD160 | RHASH_GOST | OPT_ED2K_LINK;
+		RHASH_AICH | RHASH_WHIRLPOOL | RHASH_RIPEMD160 | RHASH_GOST12_256 | OPT_ED2K_LINK;
 	char* short_opt = "cmhteawrgl";
 	print_hash_info *info = hash_info_table;
 	unsigned fullmask = RHASH_ALL_HASHES | OPT_ED2K_LINK;
 
 	memset(hash_info_table, 0, sizeof(hash_info_table));
 
-	for (index = 0, bit = 1; bit <= fullmask; index++, bit = bit << 1, info++) {
+	for (bit = 1; bit && bit <= fullmask; bit = bit << 1) {
 		const char *p;
 		char *e, *d;
 
+		if (!(bit & fullmask))
+			continue;
+
 		info->short_char = ((bit & short_opt_mask) != 0 && *short_opt ?
 			*(short_opt++) : 0);
 
 		info->name = (bit & RHASH_ALL_HASHES ? rhash_get_name(bit) : "ED2K-LINK");
-		assert(strlen(info->name) < 15);
+		assert(strlen(info->name) < 19);
 		p = info->name;
 		d = info->short_name;
-		e = info->short_name + 15; /* buffer overflow protection */
+		e = info->short_name + 19; /* buffer overflow protection */
 
-		if (memcmp(info->name, "SHA", 3) == 0) {
+		if (memcmp(info->name, "SHA", 3) == 0 || memcmp(info->name, "GOST", 4) == 0) {
 			strcpy(d, p);
 			for (; *d && d < e; d++) {
 				if ('A' <= *d && *d <= 'Z') {
@@ -551,6 +554,7 @@ void init_hash_info_table(void)
 			}
 		}
 		*d = 0;
+		++info;
 	}
 }
 
@@ -603,7 +607,7 @@ void init_printf_format(strbuf_t* out)
 	}
 
 	/* loop by hashes */
-	for (bit = 1 << index; bit <= opt.sum_flags; bit = bit << 1, index++) {
+	for (bit = 1 << index; bit && bit <= opt.sum_flags; bit = bit << 1, index++) {
 		const char *p;
 		print_hash_info *info;
 
@@ -669,12 +673,8 @@ int print_sfv_header_line(FILE* out, file_t* file, const char* printpath)
 	/* skip stdin stream */
 	if ((file->mode & FILE_IFSTDIN) != 0) return 0;
 
-#ifdef _WIN32
 	/* skip file if it can't be opened with exclusive sharing rights */
-	if (!can_open_exclusive(file->path)) {
-		return 0;
-	}
-#endif
+	if (file_is_write_locked(file)) return 0;
 
 	if (!printpath) printpath = file->path;
 	if (printpath[0] == '.' && IS_PATH_SEPARATOR(printpath[1])) printpath += 2;
