@@ -2,7 +2,7 @@
 
 #include <errno.h>
 #include <stdio.h>
-#include <stdlib.h> /* for qsort */
+#include <stdlib.h>
 #include <string.h>
 #ifndef _WIN32
 # include <dirent.h>
