@@ -90,12 +90,9 @@ void log_file_error(const char* filepath)
  *
  * @param file the file, caused the error
  */
-void log_file_t_error(const struct file_t* file)
+void log_file_t_error(struct file_t* file)
 {
-#ifdef _WIN32
-	if (!file->path) log_file_error(w2c(file->wpath));
-#endif
-	log_file_error(file->path);
+	log_file_error(file_cpath(file));
 }
 
 /**
@@ -479,7 +476,8 @@ void print_check_stats(void)
 		/* NOTE: don't use puts() here cause it mess with printf stdout buffering */
 		rsh_fprintf(rhash_data.out, _("Everything OK\n"));
 	} else {
-		rsh_fprintf(rhash_data.out, _("Errors Occurred: Errors:%-3u Miss:%-3u Success:%-3u Total:%-3u\n"), rhash_data.processed-rhash_data.ok-rhash_data.miss, rhash_data.miss, rhash_data.ok, rhash_data.processed);
+		rsh_fprintf(rhash_data.out, _("Errors Occurred: Errors:%-3u Miss:%-3u Success:%-3u Total:%-3u\n"),
+			rhash_data.processed-rhash_data.ok-rhash_data.miss, rhash_data.miss, rhash_data.ok, rhash_data.processed);
 	}
 	fflush(rhash_data.out);
 }
