@@ -2,6 +2,7 @@
 
 #include <assert.h>
 #include <locale.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -11,6 +12,7 @@
 
 #include "parse_cmdline.h"
 #include "common_func.h"
+#include "file.h"
 #include "file_mask.h"
 #include "find_file.h"
 #include "hash_print.h"
@@ -37,9 +39,12 @@ static void print_version(void)
 	rsh_exit(0);
 }
 
-static void print_help_line(const char* option, const char* text)
+static void print_help_line(const char* option, const char* format, ...)
 {
-	rsh_fprintf(rhash_data.out, "%s%s", option, text);
+	va_list args;
+	va_start(args, format);
+	rsh_fprintf(rhash_data.out, "%s", option);
+	rsh_vfprintf(rhash_data.out, format, args);
 }
 
 /**
@@ -47,34 +52,42 @@ static void print_help_line(const char* option, const char* text)
  */
 static void print_help(void)
 {
+	const char* hash_sum_format;
 	assert(rhash_data.out != NULL);
 
 	/* print program version and usage */
 	rsh_fprintf(rhash_data.out, _("%s\n"
 		"Usage: %s [OPTION...] [FILE | -]...\n"
 		"       %s --printf=<format string> [FILE | -]...\n\n"), get_full_program_version(), CMD_FILENAME, CMD_FILENAME);
-	rsh_fprintf(rhash_data.out, _("Options:\n"));
 
+	rsh_fprintf(rhash_data.out, _("Options:\n"));
 	print_help_line("  -V, --version ", _("Print program version and exit.\n"));
 	print_help_line("  -h, --help    ", _("Print this help screen.\n"));
-	print_help_line("  -C, --crc32   ", _("Calculate CRC32 hash sum.\n"));
-	print_help_line("      --md4     ", _("Calculate MD4   hash sum.\n"));
-	print_help_line("  -M, --md5     ", _("Calculate MD5   hash sum.\n"));
-	print_help_line("  -H, --sha1    ", _("Calculate SHA1  hash sum.\n"));
-	print_help_line("      --sha224, --sha256, --sha384, --sha512 ", _("Calculate SHA2 hash sum.\n"));
-	print_help_line("      --sha3-224, --sha3-256, --sha3-384, --sha3-512 ", _("Calculate SHA3 hash sum.\n"));
-	print_help_line("  -T, --tth     ", _("Calculate TTH sum.\n"));
-	print_help_line("      --btih    ", _("Calculate BitTorrent InfoHash.\n"));
-	print_help_line("  -A, --aich    ", _("Calculate AICH hash.\n"));
-	print_help_line("  -E, --ed2k    ", _("Calculate eDonkey hash sum.\n"));
+	/* TRANSLATORS: format string for a help line, like: "Calculate SHA1 hash sum.\n" */
+	hash_sum_format = _("Calculate %s hash sum.\n");
+	print_help_line("  -C, --crc32   ", hash_sum_format, "CRC32");
+	print_help_line("      --crc32c  ", hash_sum_format, "CRC32C");
+	print_help_line("      --md4     ", hash_sum_format, "MD4");
+	print_help_line("  -M, --md5     ", hash_sum_format, "MD5");
+	print_help_line("  -H, --sha1    ", hash_sum_format, "SHA1");
+	print_help_line("      --sha224, --sha256, --sha384, --sha512 ", hash_sum_format, "SHA2");
+	print_help_line("      --sha3-224, --sha3-256, --sha3-384, --sha3-512 ", hash_sum_format, "SHA3");
+	print_help_line("  -T, --tth     ", hash_sum_format, "TTH");
+	print_help_line("      --btih    ", hash_sum_format, "BitTorrent InfoHash");
+	print_help_line("  -A, --aich    ", hash_sum_format, "AICH");
+	print_help_line("  -E, --ed2k    ", hash_sum_format, "eDonkey");
 	print_help_line("  -L, --ed2k-link  ", _("Calculate and print eDonkey link.\n"));
-	print_help_line("      --tiger   ", _("Calculate Tiger hash sum.\n"));
-	print_help_line("  -G, --gost    ", _("Calculate GOST R 34.11-94 hash.\n"));
-	print_help_line("      --gost-cryptopro ", _("CryptoPro version of the GOST R 34.11-94 hash.\n"));
-	print_help_line("      --ripemd160  ", _("Calculate RIPEMD-160 hash.\n"));
-	print_help_line("      --has160  ", _("Calculate HAS-160 hash.\n"));
-	print_help_line("      --edonr256, --edonr512  ", _("Calculate EDON-R 256/512 hash.\n"));
-	print_help_line("      --snefru128, --snefru256  ", _("Calculate SNEFRU-128/256 hash.\n"));
+	print_help_line("      --tiger   ", hash_sum_format, "Tiger");
+	print_help_line("  -G, --gost12-256 ", hash_sum_format, _("GOST R 34.11-2012, 256 bit"));
+	print_help_line("      --gost12-512 ", hash_sum_format, _("GOST R 34.11-2012, 512 bit"));
+	/* TRANSLATORS: This hash function name should be translated to Russian only */
+	print_help_line("      --gost94  ", hash_sum_format, _("GOST R 34.11-94"));
+	/* TRANSLATORS: This hash function name should be translated to Russian only */
+	print_help_line("      --gost94-cryptopro ", hash_sum_format, _("GOST R 34.11-94 CryptoPro"));
+	print_help_line("      --ripemd160  ", hash_sum_format, "RIPEMD-160");
+	print_help_line("      --has160  ", hash_sum_format, "HAS-160");
+	print_help_line("      --edonr256, --edonr512  ", hash_sum_format, "EDON-R 256/512");
+	print_help_line("      --snefru128, --snefru256  ", hash_sum_format, "SNEFRU-128/256");
 	print_help_line("  -a, --all     ", _("Calculate all supported hashes.\n"));
 	print_help_line("  -c, --check   ", _("Check hash files specified by command line.\n"));
 	print_help_line("  -u, --update  ", _("Update hash files specified by command line.\n"));
@@ -85,6 +98,7 @@ static void print_help(void)
 	print_help_line("  -v, --verbose ", _("Be verbose.\n"));
 	print_help_line("  -r, --recursive  ", _("Process directories recursively.\n"));
 	print_help_line("      --file-list=<file> ", _("Process a list of files.\n"));
+	print_help_line("  -m, --message=<text> ", _("Process the text message.\n"));
 	print_help_line("      --skip-ok ", _("Don't print OK messages for successfully verified files.\n"));
 	print_help_line("  -i, --ignore-case  ", _("Ignore case of filenames when updating hash files.\n"));
 	print_help_line("      --percents   ", _("Show percents, while calculating or checking hashes.\n"));
@@ -97,14 +111,16 @@ static void print_help(void)
 	print_help_line("      --sfv     ", _("Print hash sums, using SFV format (default).\n"));
 	print_help_line("      --bsd     ", _("Print hash sums, using BSD-like format.\n"));
 	print_help_line("      --simple  ", _("Print hash sums, using simple format.\n"));
-	print_help_line("  -m, --magnet  ", _("Print hash sums  as magnet links.\n"));
+	print_help_line("  -g, --magnet  ", _("Print hash sums  as magnet links.\n"));
 	print_help_line("      --torrent ", _("Create torrent files.\n"));
 #ifdef _WIN32
-	print_help_line("      --ansi    ", _("Use Windows codepage for output (Windows only).\n"));
+	print_help_line("      --utf8    ", _("Use UTF-8 encoding for output (Windows only).\n"));
+	print_help_line("      --win     ", _("Use Windows codepage for output (Windows only).\n"));
+	print_help_line("      --dos     ", _("Use DOS codepage for output (Windows only).\n"));
 #endif
 	print_help_line("      --template=<file> ", _("Load a printf-like template from the <file>\n"));
-	print_help_line("  -p, --printf=<format string>  ",
-		_("Format and print hash sums.\n                See the RHash manual for details.\n"));
+	print_help_line("  -p, --printf=<format string>  ", _("Format and print hash sums.\n"));
+	print_help_line("                ", _("See the RHash manual for details.\n"));
 	rsh_exit(0);
 }
 
@@ -128,16 +144,16 @@ enum file_suffix_type {
 };
 
 /**
- * Add a file-list.
+ * Add a special file.
  *
  * @param o pointer to the options structure to update
- * @param path the path to the file-list
+ * @param path the path of the file
  * @param type the type of the option
  */
-static void add_file_list(options_t *o, tstr_t path, unsigned is_file_list)
+static void add_special_file(options_t *o, tstr_t path, unsigned file_mode)
 {
 	if (o->search_data) {
-		file_search_add_file(o->search_data, path, is_file_list);
+		file_search_add_file(o->search_data, path, file_mode);
 		opt.has_files = 1;
 	}
 }
@@ -313,12 +329,13 @@ enum option_type_t
 	F_UFLG = 1, /* set a bit flag in a uint32_t field */
 	F_UENC = F_UFLG | F_OUTPUT_OPT, /* an encoding changing option */
 	F_CSTR = 2 | F_NEED_PARAM, /* store parameter as a C string */
-	F_TOUT = 3 | F_NEED_PARAM | F_OUTPUT_OPT,
-	F_VFNC = 4, /* just call a function */
-	F_PFNC = 5 | F_NEED_PARAM, /* process option parameter by calling a handler */
-	F_TFNC = 6 | F_NEED_PARAM, /* process option parameter by calling a handler */
-	F_UFNC = 7 | F_NEED_PARAM, /* pass UTF-8 encoded parameter to the handler */
-	F_PRNT = 8, /* print a constant C-string and exit */
+	F_TSTR = 3 | F_NEED_PARAM, /* store parameter as a tstr_t */
+	F_TOUT = 4 | F_NEED_PARAM | F_OUTPUT_OPT,
+	F_VFNC = 5, /* just call a function */
+	F_PFNC = 6 | F_NEED_PARAM, /* process option parameter by calling a handler */
+	F_TFNC = 7 | F_NEED_PARAM, /* process option parameter by calling a handler */
+	F_UFNC = 8 | F_NEED_PARAM, /* pass UTF-8 encoded parameter to the handler */
+	F_PRNT = 9, /* print a constant C-string and exit */
 };
 
 #define is_param_required(option_type) ((option_type) & F_NEED_PARAM)
@@ -340,6 +357,7 @@ cmdline_opt_t cmdline_opt[] =
 	/* hash sums options */
 	{ F_UFLG, 'a',   0, "all",    &opt.sum_flags, RHASH_ALL_HASHES },
 	{ F_UFLG, 'C',   0, "crc32",  &opt.sum_flags, RHASH_CRC32 },
+	{ F_UFLG,   0,   0, "crc32c", &opt.sum_flags, RHASH_CRC32C },
 	{ F_UFLG,   0,   0, "md4",    &opt.sum_flags, RHASH_MD4 },
 	{ F_UFLG, 'M',   0, "md5",    &opt.sum_flags, RHASH_MD5 },
 	{ F_UFLG, 'H',   0, "sha1",   &opt.sum_flags, RHASH_SHA1 },
@@ -356,8 +374,13 @@ cmdline_opt_t cmdline_opt[] =
 	{ F_UFLG,   0,   0, "btih",   &opt.sum_flags, RHASH_BTIH },
 	{ F_UFLG, 'E',   0, "ed2k",   &opt.sum_flags, RHASH_ED2K },
 	{ F_UFLG, 'A',   0, "aich",   &opt.sum_flags, RHASH_AICH },
-	{ F_UFLG, 'G',   0, "gost",   &opt.sum_flags, RHASH_GOST },
-	{ F_UFLG,   0,   0, "gost-cryptopro", &opt.sum_flags, RHASH_GOST_CRYPTOPRO },
+	{ F_UFLG, 'G',   0, "gost12-256",   &opt.sum_flags, RHASH_GOST12_256 },
+	{ F_UFLG,   0,   0, "gost12-512",   &opt.sum_flags, RHASH_GOST12_512 },
+	{ F_UFLG,   0,   0, "gost94",   &opt.sum_flags, RHASH_GOST94 },
+	{ F_UFLG,   0,   0, "gost94-cryptopro", &opt.sum_flags, RHASH_GOST94_CRYPTOPRO },
+	/* legacy: the following two gost options are left for compatibility */
+	{ F_UFLG,   0,   0, "gost",   &opt.sum_flags, RHASH_GOST94 },
+	{ F_UFLG,   0,   0, "gost-cryptopro", &opt.sum_flags, RHASH_GOST94_CRYPTOPRO },
 	{ F_UFLG, 'W',   0, "whirlpool", &opt.sum_flags, RHASH_WHIRLPOOL },
 	{ F_UFLG,   0,   0, "ripemd160", &opt.sum_flags, RHASH_RIPEMD160 },
 	{ F_UFLG,   0,   0, "has160",    &opt.sum_flags, RHASH_HAS160 },
@@ -371,15 +394,16 @@ cmdline_opt_t cmdline_opt[] =
 	{ F_UFLG,   0,   0, "sfv",     &opt.fmt, FMT_SFV },
 	{ F_UFLG,   0,   0, "bsd",     &opt.fmt, FMT_BSD },
 	{ F_UFLG,   0,   0, "simple",  &opt.fmt, FMT_SIMPLE },
-	{ F_UFLG, 'm',   0, "magnet",  &opt.fmt, FMT_MAGNET },
+	{ F_UFLG, 'g',   0, "magnet",  &opt.fmt, FMT_MAGNET },
 	{ F_UFLG,   0,   0, "uppercase", &opt.flags, OPT_UPPERCASE },
 	{ F_UFLG,   0,   0, "lowercase", &opt.flags, OPT_LOWERCASE },
-	{ F_CSTR,   0,   0, "template",  &opt.template_file, 0 },
+	{ F_TSTR,   0,   0, "template",  &opt.template_file, 0 },
 	{ F_CSTR, 'p',   0, "printf",  &opt.printf_str, 0 },
 
 	/* other options */
 	{ F_UFLG, 'r', 'R', "recursive", &opt.flags, OPT_RECURSIVE },
-	{ F_TFNC,   0,   0, "file-list", add_file_list, 1 },
+	{ F_TFNC, 'm',   0, "message", add_special_file, FILE_IFDATA },
+	{ F_TFNC,   0,   0, "file-list", add_special_file, FILE_IFLIST },
 	{ F_UFLG,   0,   0, "follow",  &opt.flags, OPT_FOLLOW },
 	{ F_UFLG, 'v',   0, "verbose", &opt.flags, OPT_VERBOSE },
 	{ F_UFLG,   0,   0, "gost-reverse", &opt.flags, OPT_GOST_REVERSE },
@@ -401,18 +425,21 @@ cmdline_opt_t cmdline_opt[] =
 	{ F_UFLG,   0,   0, "bt-private", &opt.flags, OPT_BT_PRIVATE },
 	{ F_PFNC,   0,   0, "bt-piece-length", set_bt_piece_length, 0 },
 	{ F_UFNC,   0,   0, "bt-announce", bt_announce, 0 },
-	{ F_CSTR,   0,   0, "bt-batch", &opt.bt_batch_file, 0 },
+	{ F_TSTR,   0,   0, "bt-batch", &opt.bt_batch_file, 0 },
 	{ F_UFLG,   0,   0, "benchmark-raw", &opt.flags, OPT_BENCH_RAW },
 	{ F_PFNC,   0,   0, "openssl", openssl_flags, 0 },
 
 #ifdef _WIN32 /* code pages (windows only) */
 	{ F_UENC,   0,   0, "utf8", &opt.flags, OPT_UTF8 },
+	{ F_UENC,   0,   0, "win",  &opt.flags, OPT_ANSI },
+	{ F_UENC,   0,   0, "dos",  &opt.flags, OPT_OEM },
+	/* legacy: the following two options are left for compatibility */
 	{ F_UENC,   0,   0, "ansi", &opt.flags, OPT_ANSI },
 	{ F_UENC,   0,   0, "oem",  &opt.flags, OPT_OEM },
 #endif
 	{ 0,0,0,0,0,0 }
 };
-cmdline_opt_t cmdline_file = { F_TFNC, 0, 0, "FILE", add_file_list, 0 };
+cmdline_opt_t cmdline_file = { F_TFNC, 0, 0, "FILE", add_special_file, 0 };
 
 /**
  * Log a message and exit the program.
@@ -465,7 +492,7 @@ static void apply_option(options_t *opts, parsed_option_t* option)
 		}
 
 #ifdef _WIN32
-		if (option_type == F_TOUT || option_type == F_TFNC) {
+		if (option_type == F_TOUT || option_type == F_TFNC || option_type == F_TSTR) {
 			/* leave the value in UTF-16 */
 			value = (char*)rsh_wcsdup((wchar_t*)option->parameter);
 		}
@@ -489,6 +516,7 @@ static void apply_option(options_t *opts, parsed_option_t* option)
 		*(unsigned*)((char*)opts + ((char*)o->ptr - (char*)&opt)) |= o->param;
 		break;
 	case F_CSTR:
+	case F_TSTR:
 	case F_TOUT:
 		/* save the option parameter */
 		*(char**)((char*)opts + ((char*)o->ptr - (char*)&opt)) = value;
@@ -526,7 +554,18 @@ static const char* find_conf_file(void)
 	char *dir1, *path;
 
 #ifndef _WIN32 /* Linux/Unix part */
-	/* first check for $HOME/.rhashrc file */
+	/* first check for $XDG_CONFIG_HOME/rhash/rhashrc file */
+	if ( (dir1 = getenv("XDG_CONFIG_HOME")) ) {
+		dir1 = make_path(dir1, "rhash");
+		path = make_path(dir1, CONFIG_FILENAME);
+		free(dir1);
+		if (is_regular_file(path)) {
+			rsh_vector_add_ptr(opt.mem, path);
+			return (conf_opt.config_file = path);
+		}
+		free(path);
+	}
+	/* then check for $HOME/.rhashrc file */
 	if ( (dir1 = getenv("HOME")) ) {
 		path = make_path(dir1, ".rhashrc");
 		if (is_regular_file(path)) {
@@ -592,6 +631,7 @@ static int read_config(void)
 {
 #define LINE_BUF_SIZE 2048
 	char buf[LINE_BUF_SIZE];
+	file_t file;
 	FILE* fd;
 	parsed_option_t option;
 	int res;
@@ -602,7 +642,9 @@ static int read_config(void)
 
 	if (!find_conf_file()) return 0;
 
-	fd = fopen(conf_opt.config_file, "r");
+	file_init(&file, conf_opt.config_file, FILE_OPT_DONT_FREE_PATH);
+	fd = file_fopen(&file, FOpenRead);
+	file_cleanup(&file);
 	if (!fd) return -1;
 
 	while (fgets(buf, LINE_BUF_SIZE, fd)) {
@@ -919,7 +961,8 @@ static void set_default_sums_flags(const char* progName)
 	/* convert progName to lowercase */
 	buf = str_tolower(progName);
 
-	if (strstr(buf, "crc32")) res |= RHASH_CRC32;
+	if (strstr(buf, "crc32c")) res |= RHASH_CRC32C;
+	else if (strstr(buf, "crc32")) res |= RHASH_CRC32;
 	if (strstr(buf, "md4"))   res |= RHASH_MD4;
 	if (strstr(buf, "md5"))   res |= RHASH_MD5;
 	if (strstr(buf, "sha1"))  res |= RHASH_SHA1;
@@ -935,8 +978,10 @@ static void set_default_sums_flags(const char* progName)
 	if (strstr(buf, "tth"))   res |= RHASH_TTH;
 	if (strstr(buf, "btih"))  res |= RHASH_BTIH;
 	if (strstr(buf, "aich"))  res |= RHASH_AICH;
-	if (strstr(buf, "gost"))  res |= RHASH_GOST;
-	if (strstr(buf, "gost-cryptopro"))  res |= RHASH_GOST_CRYPTOPRO;
+	if (strstr(buf, "gost12-256"))  res |= RHASH_GOST12_256;
+	if (strstr(buf, "gost12-512"))  res |= RHASH_GOST12_512;
+	if (strstr(buf, "gost94-cryptopro"))  res |= RHASH_GOST94_CRYPTOPRO;
+	else if (strstr(buf, "gost94"))  res |= RHASH_GOST94;
 	if (strstr(buf, "has160"))  res |= RHASH_HAS160;
 	if (strstr(buf, "ripemd160"))  res |= RHASH_RIPEMD160;
 	if (strstr(buf, "whirlpool"))  res |= RHASH_WHIRLPOOL;
@@ -954,7 +999,8 @@ static void set_default_sums_flags(const char* progName)
 
 	/* change program flags only if opt.sum_flags was not set */
 	if (!opt.sum_flags) {
-		opt.sum_flags = (res ? res : (opt.fmt == FMT_MAGNET ? RHASH_TTH | RHASH_ED2K | RHASH_AICH : RHASH_CRC32));
+		opt.sum_flags = (res ? res : (opt.fmt == FMT_MAGNET ? RHASH_TTH | RHASH_ED2K | RHASH_AICH :
+			(!(opt.mode & MODE_CHECK) ? RHASH_CRC32 : 0)));
 	}
 }
 
